@@ -1,5 +1,5 @@
 // Menu.cc for FbTk - Fluxbox Toolkit 
-// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Basemenu.cc for blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Menu.cc 3844 2005-01-15 13:35:48Z fluxgen $
+// $Id: Menu.cc 3993 2005-05-06 15:22:24Z simonb $
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -86,7 +86,6 @@ Menu::Menu(MenuTheme &tm, ImageControl &imgctrl):
     m_alignment(ALIGNDONTCARE),
     m_active_index(-1),
     m_need_update(true) {
-
     // setup timers
 
     RefCount<Command> show_cmd(new SimpleCommand<Menu>(*this, &Menu::openSubmenu));
@@ -122,9 +121,7 @@ Menu::Menu(MenuTheme &tm, ImageControl &imgctrl):
 
     menu.frame_pixmap =
         menu.title_pixmap =
-        menu.hilite_pixmap =
-        menu.sel_pixmap = None;
-
+        menu.hilite_pixmap = None;
 
     menu.item_w = menu.frame_h =
         theme().titleFont().height() + theme().bevelWidth() * 2;
@@ -158,6 +155,7 @@ Menu::Menu(MenuTheme &tm, ImageControl &imgctrl):
                                 true); // save under
 
     evm.add(*this, menu.title);
+    menu.title.setRenderer(*this);
 
     event_mask |= PointerMotionMask;
     menu.frame = FbTk::FbWindow(menu.window, // parent
@@ -167,6 +165,7 @@ Menu::Menu(MenuTheme &tm, ImageControl &imgctrl):
                                 false,  // override redirect
                                 true); // save under
     evm.add(*this, menu.frame);
+    menu.frame.setRenderer(*this);
 
     menu.title.raise();
 
@@ -191,9 +190,6 @@ Menu::~Menu() {
     if (menu.hilite_pixmap)
         m_image_ctrl.removeImage(menu.hilite_pixmap);
 
-    if (menu.sel_pixmap)
-        m_image_ctrl.removeImage(menu.sel_pixmap);
-
     if (s_focused == this)
         s_focused = 0;
 }
@@ -277,6 +273,9 @@ void Menu::lower() {
 }
 
 void Menu::nextItem() {
+    if (menuitems.empty())
+        return;
+
     int old_which_press = m_which_press;
     m_active_index = -1;
     if (validIndex(old_which_press) && 
@@ -286,9 +285,7 @@ void Menu::nextItem() {
             // since it might hide the parent if we use Menu::hide
             menuitems[old_which_press]->submenu()->internal_hide();
         }
-        drawItem(old_which_press, 
-                 true,  // clear
-                 true); // transp
+        clearItem(old_which_press);
     }
 
     // restore old in case we changed m_which_press
@@ -309,13 +306,13 @@ void Menu::nextItem() {
 
     m_active_index = m_which_press;    
 
-    drawItem(m_which_press, 
-             true,  // clear
-             true); // transp
+    clearItem(m_which_press);
     
 }
 
 void Menu::prevItem() {
+    if (menuitems.empty())
+        return;
 
     int old_which_press = m_which_press;
     m_active_index = -1;
@@ -325,9 +322,7 @@ void Menu::prevItem() {
             // since it might hide the parent if we use Menu::hide
             menuitems[old_which_press]->submenu()->internal_hide();            
         }
-        drawItem(old_which_press, 
-                 true,  // clear
-                 true); // transp
+        clearItem(old_which_press);
     }
     // restore old in case we changed m_which_press
     m_which_press = old_which_press - 1;
@@ -347,9 +342,7 @@ void Menu::prevItem() {
 
     m_active_index = m_which_press;
 
-    drawItem(m_which_press, 
-             true,  // clear
-             true); // transp
+    clearItem(m_which_press);
 
 }
 
@@ -379,9 +372,7 @@ void Menu::enterParent() {
         submenu->internal_hide();
 
     m_active_index = -1;
-    drawItem(m_which_press, 
-             true,  // clear
-             true); // transp
+    //clearItem(m_which_press);
     m_which_press = -1; // dont select any in this 
     // hide self
     m_visible = false;
@@ -402,7 +393,6 @@ void Menu::updateMenu(int active_index) {
     if (m_title_vis) {
         menu.item_w = theme().titleFont().textWidth(menu.label.c_str(),
                                                     menu.label.size());
-		
         menu.item_w += (theme().bevelWidth() * 2);
     } else
         menu.item_w = 1;
@@ -453,78 +443,17 @@ void Menu::updateMenu(int active_index) {
     if (new_height < 1)
         new_height = 1;
 
+    // must update main window size whether visible or not
+    // the rest can wait until the end
     menu.window.resize(new_width, new_height);
 
-    Pixmap tmp = 0;
-    if (m_title_vis && m_need_update) {
-        tmp = menu.title_pixmap;
-        const FbTk::Texture &tex = theme().titleTexture();
-        if (!tex.usePixmap()) {
-            menu.title_pixmap = None;
-        } else {
-            menu.title_pixmap =
-                m_image_ctrl.renderImage(width(), theme().titleHeight(), tex);
-        }
+    if (!m_visible)
+        return;
 
-        if (tmp) 
-            m_image_ctrl.removeImage(tmp);
+    if (menu.frame.alpha() != alpha())
+        menu.frame.setAlpha(alpha());
 
-        // if new size of title doesn't match our
-        // buffer pixmap -> resize buffer pixmap
-        if (m_title_pm.width() != width() || 
-            m_title_pm.height() != theme().titleHeight()) {
-            m_title_pm = FbPixmap(menu.title.window(),
-                                  width(), theme().titleHeight(),
-                                  menu.title.depth());
-            m_real_title_pm = FbPixmap(menu.title.window(),
-                                       width(), theme().titleHeight(),
-                                       menu.title.depth());
-            // set pixmap that we have as real face to the user
-            menu.title.setBackgroundPixmap(m_real_title_pm.drawable());
-            menu.title.setBufferPixmap(m_real_title_pm.drawable());
-            //!! TODO: error checking?
-            GContext def_gc(menu.title);
-            if (menu.title_pixmap == 0) {
-                def_gc.setForeground(theme().titleTexture().color());
-                m_title_pm.fillRectangle(def_gc.gc(),
-                                         0, 0,
-                                         m_title_pm.width(), m_title_pm.height());
-                m_real_title_pm.fillRectangle(def_gc.gc(),
-                                              0, 0,
-                                              m_title_pm.width(), m_title_pm.height());
-            } else {
-                m_title_pm.copyArea(menu.title_pixmap, 
-                                    def_gc.gc(),
-                                    0, 0,
-                                    0, 0,
-                                    m_title_pm.width(), m_title_pm.height());
-
-                m_real_title_pm.copyArea(menu.title_pixmap, 
-                                         def_gc.gc(),
-                                         0, 0,
-                                         0, 0,
-                                         m_title_pm.width(), m_title_pm.height());
-
-            }
-
-        }
-    }
-
-    tmp = menu.frame_pixmap;
-    const FbTk::Texture &frame_tex = theme().frameTexture();
-    if (m_need_update) {
-        if (!frame_tex.usePixmap()) {
-            menu.frame_pixmap = None;
-        } else {
-            menu.frame_pixmap =
-                m_image_ctrl.renderImage(width(), menu.frame_h, frame_tex);        
-        }
-
-        if (tmp)
-            m_image_ctrl.removeImage(tmp);
-
-    }
-    tmp = menu.hilite_pixmap;
+    Pixmap tmp = menu.hilite_pixmap;
     const FbTk::Texture &hilite_tex = theme().hiliteTexture();
     if (!hilite_tex.usePixmap()) {
         menu.hilite_pixmap = None;
@@ -534,18 +463,16 @@ void Menu::updateMenu(int active_index) {
     if (tmp)
         m_image_ctrl.removeImage(tmp);
 
-    tmp = menu.sel_pixmap;
-    if (!hilite_tex.usePixmap()) {
-        menu.sel_pixmap = None;
-    } else {
+
+    if (!theme().selectedPixmap().pixmap().drawable()) {
         int hw = theme().itemHeight() / 2;
-        menu.sel_pixmap =
-            m_image_ctrl.renderImage(hw, hw, hilite_tex);
+        m_theme.setSelectedPixmap(m_image_ctrl.renderImage(hw, hw, theme().hiliteTexture()));
+
+        if (!theme().highlightSelectedPixmap().pixmap().drawable()) {
+            int hw = theme().itemHeight() / 2;
+            m_theme.setHighlightSelectedPixmap(m_image_ctrl.renderImage(hw, hw, theme().frameTexture()));
+        }
     }
-    if (tmp) 
-        m_image_ctrl.removeImage(tmp);
-
-
 
     if (m_title_vis) {
         menu.title.moveResize(-menu.title.borderWidth(), -menu.title.borderWidth(), 
@@ -553,80 +480,57 @@ void Menu::updateMenu(int active_index) {
     }
 
     menu.frame.moveResize(0, ((m_title_vis) ? menu.title.y() + menu.title.height() + 
-                              menu.title.borderWidth()*2 : 1), 
+                              menu.title.borderWidth()*2 : 0), 
                           width(), menu.frame_h);
 
-
-    if (m_need_update || m_frame_pm.width() != menu.frame.width() ||
-        m_frame_pm.height() != menu.frame.height()){
-
-        m_frame_pm = FbTk::FbPixmap(menu.frame.window(),
-                                    menu.frame.width(), menu.frame.height(),
-                                    menu.frame.depth());
-
-        m_real_frame_pm = FbTk::FbPixmap(menu.frame.window(),
-                                         menu.frame.width(), menu.frame.height(),
-                                         menu.frame.depth());
-        if (m_transp.get() != 0) 
-            m_transp->setDest(m_real_frame_pm.drawable(), screenNumber());
-
-        menu.frame.setBackgroundPixmap(m_real_frame_pm.drawable());
-        GContext def_gc(menu.frame);
-        if (m_frame_pm.drawable() == 0) {
-            _FB_USES_NLS;
-            cerr<<"FbTk::Menu: "<<_FBTKTEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<" ("<<
-                menu.frame.window()<<", "<<menu.frame.width()<<", "<<
-                menu.frame.height()<<
-                ", "<<menu.frame.depth()<<") !"<<endl;
+    if (m_title_vis && m_need_update) {
+        tmp = menu.title_pixmap;
+        const FbTk::Texture &tex = theme().titleTexture();
+        if (!tex.usePixmap()) {
+            menu.title_pixmap = None;
+            menu.title.setBackgroundColor(tex.color());
         } else {
-
-            if (menu.frame_pixmap == 0) {
-                def_gc.setForeground(theme().frameTexture().color());
-                m_frame_pm.fillRectangle(def_gc.gc(),
-                                         0, 0,
-                                         width(), menu.frame_h);
-            } else {
-                m_frame_pm.copyArea(menu.frame_pixmap, def_gc.gc(),
-                                    0, 0,
-                                    0, 0,
-                                    width(), menu.frame_h);
-            }
-            
-
+            menu.title_pixmap =
+                m_image_ctrl.renderImage(width(), theme().titleHeight(), tex);
+            // set pixmap that we have as real face to the user
+            menu.title.setBackgroundPixmap(menu.title_pixmap);
         }
 
-        m_real_frame_pm.copyArea(m_frame_pm.drawable(),
-                                 def_gc.gc(),
-                                 0, 0,
-                                 0, 0,
-                                 m_frame_pm.width(), m_frame_pm.height());
-            
+        if (tmp) 
+            m_image_ctrl.removeImage(tmp);
     }
 
-    // if menu m_visible and title m_visible
-    if (m_title_vis && m_visible) 
-        redrawTitle();
-
-    if (active_index >= 0 || m_need_update) {
-
-        renderTransp(0, 0,
-                     m_real_frame_pm.width(), m_real_frame_pm.height());
-        for (unsigned int i = 0; i < menuitems.size(); i++) {
-            drawItem(i,      // index
-                     true,   // clear
-                     false); // render_trans
+    tmp = menu.frame_pixmap;
+    const FbTk::Texture &frame_tex = theme().frameTexture();
+    if (m_need_update) {
+        if (!frame_tex.usePixmap()) {
+            menu.frame_pixmap = None;
+            menu.frame.setBackgroundColor(frame_tex.color());
+        } else {
+            menu.frame_pixmap =
+                m_image_ctrl.renderImage(width(), menu.frame_h, frame_tex);        
+            menu.frame.setBackgroundPixmap(menu.frame_pixmap);
         }
-        
+
+        if (tmp)
+            m_image_ctrl.removeImage(tmp);
+
     }
 
+    clearWindow();
     m_need_update = false;
 }
 
 
 void Menu::show() {
 
-    if (m_need_update)
+    if (m_visible)
+        return;
+
+    if (m_need_update) {
+        m_visible = true;
         updateMenu();
+    }
 
     menu.window.showSubwindows();
     menu.window.show();
@@ -672,16 +576,20 @@ void Menu::grabInputFocus() {
 
 
 void Menu::clearWindow() {
-    redrawTitle();
- 
-    if (alpha() < 255) {
-        renderTransp(0, 0,
-                     menu.frame.width(), menu.frame.height());
-        updateMenu();
-    }
-
     menu.title.clear();
     menu.frame.clear();
+    
+    // clear foreground bits of frame items
+    for (unsigned int i = 0; i < menuitems.size(); i++) {
+        clearItem(i, false);   // no clear
+    }
+}
+
+void Menu::redrawFrame(FbDrawable &drawable) {
+    for (unsigned int i = 0; i < menuitems.size(); i++) {
+        drawItem(drawable, i);
+    }
+
 }
 
 void Menu::internal_hide() {
@@ -695,14 +603,14 @@ void Menu::internal_hide() {
     // as non active
     int old = m_active_index;
     m_active_index = -1;
-    drawItem(old, true); // clear old area from highlight
+    clearItem(old); // clear old area from highlight
 
     if (shown && shown->menu.window == menu.window)
         shown = (Menu *) 0;
 
     m_torn = m_visible = false;
     m_which_sub = m_which_press = m_which_sub = -1;
-    
+
     menu.window.hide();
 }
 
@@ -711,37 +619,23 @@ void Menu::move(int x, int y) {
     if (x == this->x() && y == this->y())
         return;
 
-    // if we're not m_visible and we do transparency
-    // we need to update transparency when we call
-    // show() next time
-    if (alpha() < 255)
-        m_need_update = true;
-
     menu.window.move(x, y);
+    // potentially transparent children
+    menu.title.parentMoved();
+    menu.frame.parentMoved();
 
     if (!isVisible())
         return;
 
+    if (alpha() < 255)
+        clearWindow();
+
     if (m_which_sub != -1)
         drawSubmenu(m_which_sub);
-
-    if (alpha() < 255) {
-        redrawTitle();
-        menu.title.clear();
-        renderTransp(0, 0,
-                     m_real_frame_pm.width(), m_real_frame_pm.height());
-        for (size_t i=0; i < menuitems.size(); ++i) {
-            drawItem(i,
-                     true, // clear
-                     false); // transparent
-        }
-        m_need_update = false;
-    }
-
 }
 
 
-void Menu::redrawTitle() {
+void Menu::redrawTitle(FbDrawable &drawable) {
     const char *text = menu.label.c_str();
 
     const FbTk::Font &font = theme().titleFont();
@@ -760,21 +654,9 @@ void Menu::redrawTitle() {
         break;
     }
 
-    if (menu.title.alpha() != alpha())
-        menu.title.setAlpha(alpha());
-
-    FbTk::GContext def_gc(menu.title);
-
-    m_real_title_pm.copyArea(m_title_pm.drawable(),
-                             def_gc.gc(),
-                             0, 0,
-                             0, 0,
-                             m_title_pm.width(), m_title_pm.height());
-
     // difference between height based on font, and style-set height
     int height_offset = theme().titleHeight() - (font.height() + 2*theme().bevelWidth());
-    menu.title.updateTransparent();
-    font.drawText(m_real_title_pm, // drawable
+    font.drawText(drawable, // drawable
                   screenNumber(),
                   theme().titleTextGC().gc(), // graphic context
                   text, len,  // text string with length
@@ -849,7 +731,7 @@ void Menu::drawSubmenu(unsigned int index) {
 
         item->submenu()->move(new_x, new_y);
         if (! m_moving)
-            drawItem(index);
+            clearItem(index);
 		
         if (! item->submenu()->isVisible()) {
             item->submenu()->show();
@@ -874,8 +756,9 @@ bool Menu::hasSubmenu(unsigned int index) const {
 }
 
 
-int Menu::drawItem(unsigned int index, bool clear, bool render_trans,
-                   int x, int y, unsigned int w, unsigned int h) {
+int Menu::drawItem(FbDrawable &drawable, unsigned int index,
+                   bool highlight, bool exclusive_drawable) {
+
     if (index >= menuitems.size() || menuitems.size() == 0 ||
         menu.persub == 0)
         return 0;
@@ -885,120 +768,15 @@ int Menu::drawItem(unsigned int index, bool clear, bool render_trans,
 
     int sbl = index / menu.persub, i = index - (sbl * menu.persub);
     int item_x = (sbl * menu.item_w), item_y = (i * theme().itemHeight());
-    int hilite_x = item_x, hilite_y = item_y, hoff_x = 0, hoff_y = 0;
-    int sel_x = 0, sel_y = 0;
-    unsigned int hilite_w = menu.item_w, hilite_h = theme().itemHeight();
-    unsigned int half_w = theme().itemHeight() / 2, quarter_w = theme().itemHeight() / 4;
-    bool highlight = (index == m_active_index);
-    GC gc =
-        ((highlight || item->isSelected()) ? theme().hiliteTextGC().gc() :
-         theme().frameTextGC().gc());
-	
-    sel_x = item_x;
-	
-    if (theme().bulletPos() == FbTk::RIGHT)
-        sel_x += (menu.item_w - theme().itemHeight() - theme().bevelWidth());
-	
-    sel_x += quarter_w;
-    sel_y = item_y + quarter_w;
 
-    if (clear) {
-        GContext def_gc(menu.frame);
-        if (menu.frame_pixmap == 0) {
-            def_gc.setForeground(theme().frameTexture().color());
-            m_frame_pm.fillRectangle(def_gc.gc(), item_x, item_y, menu.item_w, theme().itemHeight());
+    if (exclusive_drawable)
+        item_x = item_y = 0;
 
-        } else {
-
-            m_frame_pm.copyArea(menu.frame_pixmap, def_gc.gc(),
-                                item_x, item_y,
-                                item_x, item_y,
-                                menu.item_w, theme().itemHeight());
-        }    
-    } else if (! (x == y && y == -1 && w == h && h == 0)) {
-        // calculate the which part of the hilite to redraw
-        if (!(std::max(item_x, x) <= (signed) std::min(item_x + menu.item_w, x + w) &&
-              std::max(item_y, y) <= (signed) std::min(item_y + theme().itemHeight(), y + h))) {
-            hilite_x = std::max(item_x, x);
-            hilite_y = std::max(item_y, y);
-            hilite_w = std::min(item_x + menu.item_w, x + w) - hilite_x;
-            hilite_h = std::min(item_y + theme().itemHeight(), y + h) - hilite_y;
-            hoff_x = hilite_x % menu.item_w;
-            hoff_y = hilite_y % theme().itemHeight();
-        }
-		
-    }
-    
-    if (highlight && (menu.hilite_pixmap != ParentRelative)) {
-        if (menu.hilite_pixmap) {
-            m_frame_pm.copyArea(menu.hilite_pixmap,
-                                theme().hiliteGC().gc(), hoff_x, hoff_y,
-                                hilite_x, hilite_y,
-                                hilite_w, hilite_h);
-        } else {            
-            m_frame_pm.fillRectangle(theme().hiliteGC().gc(),
-                                     hilite_x, hilite_y, hilite_w, hilite_h);
-        }
-        
-    } 
-	
-
-    if (render_trans) {
-        renderTransp(item_x, item_y,
-                     menu.item_w, theme().itemHeight()); 
-    }
-    //!!
-    //!! TODO: Move this out to MenuItem
-    //!! current problem: menu.sel_pixmap needs a image control instance
-    //!! to be generated :(
-    //!!
-    if (item->isToggleItem() && item->isSelected()) {
-        Display *disp = FbTk::App::instance()->display();
-        if (theme().selectedPixmap().pixmap().drawable()) {
-
-            // enable clip mask
-            XSetClipMask(disp,
-                         gc,
-                         theme().selectedPixmap().mask().drawable());
-            XSetClipOrigin(disp,
-                           gc, sel_x, item_y);
-            // copy bullet pixmap to frame
-            m_real_frame_pm.copyArea(theme().selectedPixmap().pixmap().drawable(),
-                                     gc,
-                                     0, 0,
-                                     sel_x, item_y,
-                                     theme().selectedPixmap().width(),
-                                     theme().selectedPixmap().height());
-            // disable clip mask
-            XSetClipMask(disp,
-                         gc,
-                         None);
-        } else {
-            if (menu.sel_pixmap) {
-                m_real_frame_pm.copyArea(highlight ? menu.frame_pixmap : menu.sel_pixmap,
-                                         theme().hiliteGC().gc(), 
-                                         0, 0,                                 
-                                         sel_x, sel_y,
-                                         half_w, half_w);
-            } else {
-                m_real_frame_pm.fillRectangle(theme().hiliteGC().gc(),
-                                              sel_x, sel_y, half_w, half_w);
-            }
-        }
-        
-    }
-
-    item->draw(m_real_frame_pm, theme(), highlight, 
+    item->draw(drawable, theme(), highlight, 
+               exclusive_drawable, true, // draw fg, draw bg
                item_x, item_y, 
                menu.item_w, theme().itemHeight());
 
-
-    if (clear) {
-        frameWindow().clearArea(item_x, item_y,
-                                menu.item_w, theme().itemHeight(), False);
-    }
-
-
     return item_y;
 }
 
@@ -1057,10 +835,9 @@ bool Menu::isItemSelectable(unsigned int index) const {
     if (index >= menuitems.size()) return false;
 
     const MenuItem *item = find(index);
-    if (!item)
-        return false;
-
-    return (typeid(*item) != typeid(FbTk::MenuSeparator));
+    return (!item || 
+        (typeid(*item) == typeid(FbTk::MenuSeparator)) || 
+        !item->isEnabled()) ? false : true;
 }
 
 
@@ -1092,12 +869,7 @@ void Menu::buttonPressEvent(XButtonEvent &be) {
             if (item->submenu()) {
                 if (!item->submenu()->isVisible())
                     drawSubmenu(w);
-            } else {
-                drawItem(w, 
-                         true,  // clear
-                         true); // render transparency
             }
-
         }
     } else {
         menu.x_move = be.x_root - x();
@@ -1115,16 +887,10 @@ void Menu::buttonReleaseEvent(XButtonEvent &re) {
                 drawSubmenu(m_which_sub);
 
             if (alpha() < 255) {
-                redrawTitle();
-                menu.title.clear();
-                renderTransp(0, 0,
-                             m_real_frame_pm.width(), m_real_frame_pm.height());
-                for (size_t i=0; i < menuitems.size(); ++i) {
-                    drawItem(i, 
-                             true, // clear
-                             false); // transparent
-                }
-
+                // update these since we've (probably) moved
+                menu.title.parentMoved();
+                menu.frame.parentMoved();
+                clearWindow();
             }
         }
 
@@ -1147,9 +913,7 @@ void Menu::buttonReleaseEvent(XButtonEvent &re) {
                 }
             }
 
-            drawItem(p,
-                     true,  // clear
-                     true); // transparent
+            clearItem(p);
         }
     }
 }
@@ -1163,13 +927,13 @@ void Menu::motionNotifyEvent(XMotionEvent &me) {
         if (! m_moving) {
             // if not m_moving: start m_moving operation
             if (m_parent && (! m_torn)) {
-                m_parent->drawItem(m_parent->m_which_sub, 
-                                   true,  // clear
-                                   true); // render transparency
                 m_parent->m_which_sub = -1;
+                m_parent->clearItem(m_parent->m_which_sub);  // clear
             }
 
             m_moving = m_torn = true;
+            // clear current highlighted item
+            clearItem(m_active_index);
 
             if (m_which_sub >= 0)
                 drawSubmenu(m_which_sub);
@@ -1196,12 +960,9 @@ void Menu::motionNotifyEvent(XMotionEvent &me) {
 
             if (item != 0) {
 
-                drawItem(old,
-                         true,  // clear
-                         true); // transparent
+                clearItem(old);
 
                 if (item->submenu()) {
-
                     if (item->submenu()->isVisible() &&
                         !item->submenu()->isTorn()) {
                         // setup hide timer for submenu
@@ -1236,9 +997,8 @@ void Menu::motionNotifyEvent(XMotionEvent &me) {
             // draw item highlighted and
             // start submenu open delay
 
-            drawItem(w, 
-                     true); // clear
-            
+            clearItem(w);
+
             if (theme().menuMode() == MenuTheme::DELAY_OPEN) {
                 // setup show menu timer
                 timeval timeout;
@@ -1254,9 +1014,7 @@ void Menu::motionNotifyEvent(XMotionEvent &me) {
             // draw highlighted
             m_submenu_timer.stop();
             if (itmp->isEnabled()) {
-                drawItem(w, 
-                         true,  // clear
-                         true); // transp
+                clearItem(w);
             }
         }
         
@@ -1266,10 +1024,37 @@ void Menu::motionNotifyEvent(XMotionEvent &me) {
 
 void Menu::exposeEvent(XExposeEvent &ee) {
     if (ee.window == menu.title) {
-        redrawTitle();
         menu.title.clearArea(ee.x, ee.y, ee.width, ee.height);
-    } else if (ee.window == menu.frame)
-        menu.frame.clearArea(ee.x, ee.y, ee.width, ee.height);
+    } else if (ee.window == menu.frame) {
+        // find where to clear
+        // this is a compilicated algorithm... lets do it step by step...
+        // first... we see in which sub level the expose starts... and how many
+        // items down in that sublevel
+
+        int sbl = (ee.x / menu.item_w), id = (ee.y / theme().itemHeight()),
+            // next... figure out how many sublevels over the redrawspans
+            sbl_d = ((ee.x + ee.width) / menu.item_w),
+            // then we see how many items down to redraw
+            id_d = ((ee.y + ee.height) / theme().itemHeight());
+
+        if (id_d > menu.persub) id_d = menu.persub;
+        // draw the sublevels and the number of items the exposure spans
+        int i, ii;
+        for (i = sbl; i <= sbl_d; i++) {
+            // set the iterator to the first item in the sublevel needing redrawing
+            int index = id + i * menu.persub;
+
+            if (index < static_cast<int>(menuitems.size()) && index >= 0) {
+                Menuitems::iterator it = menuitems.begin() + index;
+                Menuitems::iterator it_end = menuitems.end();
+                for (ii = id; ii <= id_d && it != it_end; ++it, ii++) {
+                    int index = ii + (i * menu.persub);
+                    // redraw the item
+                    clearItem(index);
+                }
+            }
+        }
+    }
 }
 
 
@@ -1296,9 +1081,13 @@ void Menu::enterNotifyEvent(XCrossingEvent &ce) {
     }
 
 
-    if (m_shifted)
+    if (m_shifted) {
         menu.window.move(menu.x_shift, menu.y_shift);
-    
+        menu.title.parentMoved();
+        menu.frame.parentMoved();
+        clearWindow();
+    }
+
     if (validIndex(m_which_sub)) {
         MenuItem *tmp = menuitems[m_which_sub];
         if (tmp->submenu()->isVisible()) {
@@ -1308,9 +1097,7 @@ void Menu::enterNotifyEvent(XCrossingEvent &ce) {
             if (w != m_which_sub && (! tmp->submenu()->isTorn())) {
                 tmp->submenu()->internal_hide();
 
-                drawItem(m_which_sub, 
-                         true,  // clear
-                         true); // transp
+                clearItem(m_which_sub); // not highlighted anymore
                 m_which_sub = -1;
             }
         }
@@ -1321,20 +1108,6 @@ void Menu::leaveNotifyEvent(XCrossingEvent &ce) {
     if (menu.frame != ce.window)
         return;
 
-    if (m_which_press != -1 && m_which_sbl != -1 && menuitems.size() > 0) {
-        int p = (m_which_sbl * menu.persub) + m_which_press;
-
-        drawItem(p, 
-                 true,  // clear
-                 true); // transp
-
-        m_which_sbl = m_which_press = -1;
-    }
-
-    if (m_shifted) {
-        //        menu.window.move(menu.x, menu.y);
-        m_shifted = false;
-    }
 }
 
 void Menu::keyPressEvent(XKeyEvent &event) {
@@ -1379,31 +1152,13 @@ void Menu::keyPressEvent(XKeyEvent &event) {
 
 void Menu::reconfigure() {
 
-    if (FbTk::Transparent::haveComposite()) {
-        if (m_transp.get() != 0)
-            m_transp.reset(0);
-
+    if (FbTk::Transparent::haveComposite())
         menu.window.setOpaque(alpha());
-    } else if (alpha() == 255 && m_transp.get() != 0) {
-        m_transp.reset(0);
-    } else if (alpha () < 255) {
-        if (m_transp.get() == 0) {
-            m_transp.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()),
-                                           m_real_frame_pm.drawable(), alpha(),
-                                           screenNumber()));
-        } else {
-            Pixmap root = FbPixmap::getRootPixmap(screenNumber());
-            if (m_transp->source() != root)
-                m_transp->setSource(root, screenNumber());
-
-            m_transp->setAlpha(alpha());
-        }
-    }
 
     m_need_update = true; // redraw items
 
-
     menu.title.setAlpha(alpha());
+    menu.frame.setAlpha(alpha());
 
     menu.window.setBorderColor(theme().borderColor());
     menu.title.setBorderColor(theme().borderColor());
@@ -1411,10 +1166,8 @@ void Menu::reconfigure() {
 
     menu.window.setBorderWidth(theme().borderWidth());
     menu.title.setBorderWidth(theme().borderWidth());
-    
 
     updateMenu();
-
 }
     
 
@@ -1427,8 +1180,7 @@ void Menu::openSubmenu() {
     if (!validIndex(item))
         return;
 
-    drawItem(item, 
-             true); // clear
+    clearItem(item);
     if (menuitems[item]->submenu() != 0 && !menuitems[item]->submenu()->isVisible())
         drawSubmenu(item);
 
@@ -1457,67 +1209,13 @@ void Menu::update(FbTk::Subject *subj) {
     
     Menuitems::iterator it = menuitems.begin();
     Menuitems::iterator it_end = menuitems.end();
-    for (; it != it_end; ++it)
+    for (; it != it_end; ++it) {
         (*it)->updateTheme(theme());
+    }
     reconfigure();
 }
 
                   
-void Menu::renderTransp(int x, int y,
-                        unsigned int width, unsigned int height) {
-    // even though we dont render transparency
-    // we do need to copy the style background
-    GContext def_gc(menu.frame);
-    m_real_frame_pm.copyArea(m_frame_pm.drawable(),
-                             def_gc.gc(),
-                             x, y,
-                             x, y,
-                             width, height);
-
-    // no need to render transparent unless visible
-    // but we do need to render it if we marked it as
-    // need update
-    if (!isVisible() && !m_need_update || m_transp.get() == 0)
-        return;
-
-
-    // render the root background
-#ifdef HAVE_XRENDER
-
-    Pixmap root = FbPixmap::getRootPixmap(screenNumber());
-    if (m_transp->source() != root)
-        m_transp->setSource(root, screenNumber());
-
-    if (m_transp->dest() != m_real_frame_pm.drawable())
-        m_transp->setDest(m_real_frame_pm.drawable(), screenNumber());
-
-    if (m_transp->alpha() != alpha())
-        m_transp->setAlpha(alpha());
-
-    const FbWindow *root_parent = menu.frame.parent();
-    // our position in parent ("root")
-    int root_x = menu.frame.x() - menu.frame.borderWidth(), 
-        root_y = menu.frame.y() - menu.frame.borderWidth();
-    if (root_parent != 0) {
-        root_x += root_parent->x() + root_parent->borderWidth();
-        root_y += root_parent->y() + root_parent->borderWidth();
-        while (root_parent->parent() != 0) {
-            root_parent = root_parent->parent();
-            root_x += root_parent->x() + root_parent->borderWidth();
-            root_y += root_parent->y() + root_parent->borderWidth();
-        }
-
-    } // else toplevel window so we already have x, y set
-
-    // render background image from root pos to our window
-    m_transp->render(root_x + x, root_y + y,
-                     x, y,
-                     width, height);
-        
-#endif // HAVE_XRENDER
-        
-}
-
 void Menu::setScreen(int x, int y, int w, int h) {
     m_screen_x = x;
     m_screen_y = y;
@@ -1525,4 +1223,73 @@ void Menu::setScreen(int x, int y, int w, int h) {
     m_screen_height = h;
 }
 
+// Render the foreground objects of given window onto given pixmap
+void Menu::renderForeground(FbWindow &win, FbDrawable &drawable) {
+    if (&win == &menu.frame) {
+        redrawFrame(drawable);
+    } else if (&win == &menu.title) {
+        redrawTitle(drawable);
+    }
+}
+
+// clear item clears the item and draws the dynamic bits
+// thus sometimes it won't perform the actual clear operation
+// nothing in here should be rendered transparently
+// (unless you use a caching pixmap, which I think we should avoid)
+void Menu::clearItem(int index, bool clear) {
+    if (!validIndex(index))
+        return;
+
+    int sbl = index / menu.persub, i = index - (sbl * menu.persub);
+    unsigned int item_w = menu.item_w, item_h = theme().itemHeight();
+    int item_x = (sbl * item_w), item_y = (i * item_h);
+    bool highlight = (index == m_active_index);
+
+    // don't highlight if moving, doesn't work with alpha on
+    if (highlight && !m_moving) {
+        highlightItem(index);
+        return;
+    } else if (clear)
+        menu.frame.clearArea(item_x, item_y, item_w, item_h);
+
+    MenuItem *item = menuitems[index];
+    if (! item) return;
+
+    item->draw(menu.frame, theme(), highlight,
+               true, false, item_x, item_y, 
+               item_w, item_h);
+}
+
+// Area must have been cleared before calling highlight
+void Menu::highlightItem(int index) {
+    int sbl = index / menu.persub, i = index - (sbl * menu.persub);
+    unsigned int item_w = menu.item_w, item_h = theme().itemHeight();
+    int item_x = (sbl * menu.item_w), item_y = (i * item_h);
+
+    FbPixmap buffer = FbPixmap(menu.frame, item_w, item_h, menu.frame.depth());
+
+    int hilite_x = item_x, hilite_y = item_y;
+    if (menu.hilite_pixmap != ParentRelative) {
+        if (menu.hilite_pixmap) {
+            buffer.copyArea(menu.hilite_pixmap,
+                            theme().hiliteGC().gc(), 0, 0,
+                            0, 0,
+                            item_w, item_h);
+        } else {
+            buffer.fillRectangle(theme().hiliteGC().gc(),
+                                 0, 0, item_w, item_h);
+        }
+        menu.frame.updateTransparent(item_x, item_y, item_w, item_h, buffer.drawable(), true);
+    }
+
+
+    drawItem(buffer, index, true, true);
+
+    menu.frame.copyArea(buffer.drawable(), theme().hiliteGC().gc(),
+                        0, 0,
+                        item_x, item_y,
+                        item_w, item_h);
+
+}
+
 }; // end namespace FbTk
